# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qm/pb/job_manager.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from qm.pb import errors_pb2 as qm_dot_pb_dot_errors__pb2
from qm.pb import general_messages_pb2 as qm_dot_pb_dot_general__messages__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='qm/pb/job_manager.proto',
  package='qm.grpc.jobManager',
  syntax='proto3',
  serialized_options=b'\n\022qm.grpc.jobManagerB\026JobManagerServiceProtoP\001\242\002\003HLW',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x17qm/pb/job_manager.proto\x12\x12qm.grpc.jobManager\x1a\x12qm/pb/errors.proto\x1a\x1cqm/pb/general_messages.proto\"r\n\x1bGetElementCorrectionRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x0e\n\x06qeName\x18\x02 \x01(\t\x12\x34\n\ncorrection\x18\x03 \x01(\x0b\x32 .qm.grpc.general_messages.Matrix\"\xa4\x01\n\x1cGetElementCorrectionResponse\x12N\n\x18jobManagerResponseHeader\x18\x01 \x01(\x0b\x32,.qm.grpc.jobManager.JobManagerResponseHeader\x12\x34\n\ncorrection\x18\x03 \x01(\x0b\x32 .qm.grpc.general_messages.Matrix\"r\n\x1bSetElementCorrectionRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x0e\n\x06qeName\x18\x02 \x01(\t\x12\x34\n\ncorrection\x18\x03 \x01(\x0b\x32 .qm.grpc.general_messages.Matrix\"\xa4\x01\n\x1cSetElementCorrectionResponse\x12N\n\x18jobManagerResponseHeader\x18\x01 \x01(\x0b\x32,.qm.grpc.jobManager.JobManagerResponseHeader\x12\x34\n\ncorrection\x18\x02 \x01(\x0b\x32 .qm.grpc.general_messages.Matrix\"\xac\x01\n\x18JobManagerResponseHeader\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\r\n\x05jobId\x18\x02 \x01(\t\x12\x32\n\x13jobManagerErrorType\x18\n \x01(\x0e\x32\x15.JobManagerErrorTypes\x12<\n\x0fjobErrorDetails\x18\x0b \x01(\x0b\x32#.qm.grpc.jobManager.JobErrorDetails\"\xb1\x01\n\x0fJobErrorDetails\x12H\n\x1djobOperationSpecificErrorType\x18\x01 \x01(\x0e\x32\x1f.JobOperationSpecificErrorTypesH\x00\x12\x36\n\x14\x63onfigQueryErrorType\x18\x02 \x01(\x0e\x32\x16.ConfigQueryErrorTypesH\x00\x12\x0f\n\x07message\x18\n \x01(\tB\x0b\n\terrorType\"[\n\x19JobOperationSpecificError\x12-\n\x04type\x18\x01 \x01(\x0e\x32\x1f.JobOperationSpecificErrorTypes\x12\x0f\n\x07message\x18\x02 \x01(\t2\x8d\x02\n\x11JobManagerService\x12{\n\x14GetElementCorrection\x12/.qm.grpc.jobManager.GetElementCorrectionRequest\x1a\x30.qm.grpc.jobManager.GetElementCorrectionResponse\"\x00\x12{\n\x14SetElementCorrection\x12/.qm.grpc.jobManager.SetElementCorrectionRequest\x1a\x30.qm.grpc.jobManager.SetElementCorrectionResponse\"\x00\x42\x34\n\x12qm.grpc.jobManagerB\x16JobManagerServiceProtoP\x01\xa2\x02\x03HLWb\x06proto3'
  ,
  dependencies=[qm_dot_pb_dot_errors__pb2.DESCRIPTOR,qm_dot_pb_dot_general__messages__pb2.DESCRIPTOR,])




_GETELEMENTCORRECTIONREQUEST = _descriptor.Descriptor(
  name='GetElementCorrectionRequest',
  full_name='qm.grpc.jobManager.GetElementCorrectionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='jobId', full_name='qm.grpc.jobManager.GetElementCorrectionRequest.jobId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='qeName', full_name='qm.grpc.jobManager.GetElementCorrectionRequest.qeName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='correction', full_name='qm.grpc.jobManager.GetElementCorrectionRequest.correction', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=97,
  serialized_end=211,
)


_GETELEMENTCORRECTIONRESPONSE = _descriptor.Descriptor(
  name='GetElementCorrectionResponse',
  full_name='qm.grpc.jobManager.GetElementCorrectionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='jobManagerResponseHeader', full_name='qm.grpc.jobManager.GetElementCorrectionResponse.jobManagerResponseHeader', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='correction', full_name='qm.grpc.jobManager.GetElementCorrectionResponse.correction', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=214,
  serialized_end=378,
)


_SETELEMENTCORRECTIONREQUEST = _descriptor.Descriptor(
  name='SetElementCorrectionRequest',
  full_name='qm.grpc.jobManager.SetElementCorrectionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='jobId', full_name='qm.grpc.jobManager.SetElementCorrectionRequest.jobId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='qeName', full_name='qm.grpc.jobManager.SetElementCorrectionRequest.qeName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='correction', full_name='qm.grpc.jobManager.SetElementCorrectionRequest.correction', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=380,
  serialized_end=494,
)


_SETELEMENTCORRECTIONRESPONSE = _descriptor.Descriptor(
  name='SetElementCorrectionResponse',
  full_name='qm.grpc.jobManager.SetElementCorrectionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='jobManagerResponseHeader', full_name='qm.grpc.jobManager.SetElementCorrectionResponse.jobManagerResponseHeader', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='correction', full_name='qm.grpc.jobManager.SetElementCorrectionResponse.correction', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=497,
  serialized_end=661,
)


_JOBMANAGERRESPONSEHEADER = _descriptor.Descriptor(
  name='JobManagerResponseHeader',
  full_name='qm.grpc.jobManager.JobManagerResponseHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='qm.grpc.jobManager.JobManagerResponseHeader.success', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='jobId', full_name='qm.grpc.jobManager.JobManagerResponseHeader.jobId', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='jobManagerErrorType', full_name='qm.grpc.jobManager.JobManagerResponseHeader.jobManagerErrorType', index=2,
      number=10, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='jobErrorDetails', full_name='qm.grpc.jobManager.JobManagerResponseHeader.jobErrorDetails', index=3,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=664,
  serialized_end=836,
)


_JOBERRORDETAILS = _descriptor.Descriptor(
  name='JobErrorDetails',
  full_name='qm.grpc.jobManager.JobErrorDetails',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='jobOperationSpecificErrorType', full_name='qm.grpc.jobManager.JobErrorDetails.jobOperationSpecificErrorType', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='configQueryErrorType', full_name='qm.grpc.jobManager.JobErrorDetails.configQueryErrorType', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='qm.grpc.jobManager.JobErrorDetails.message', index=2,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='errorType', full_name='qm.grpc.jobManager.JobErrorDetails.errorType',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=839,
  serialized_end=1016,
)


_JOBOPERATIONSPECIFICERROR = _descriptor.Descriptor(
  name='JobOperationSpecificError',
  full_name='qm.grpc.jobManager.JobOperationSpecificError',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='qm.grpc.jobManager.JobOperationSpecificError.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='qm.grpc.jobManager.JobOperationSpecificError.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1018,
  serialized_end=1109,
)

_GETELEMENTCORRECTIONREQUEST.fields_by_name['correction'].message_type = qm_dot_pb_dot_general__messages__pb2._MATRIX
_GETELEMENTCORRECTIONRESPONSE.fields_by_name['jobManagerResponseHeader'].message_type = _JOBMANAGERRESPONSEHEADER
_GETELEMENTCORRECTIONRESPONSE.fields_by_name['correction'].message_type = qm_dot_pb_dot_general__messages__pb2._MATRIX
_SETELEMENTCORRECTIONREQUEST.fields_by_name['correction'].message_type = qm_dot_pb_dot_general__messages__pb2._MATRIX
_SETELEMENTCORRECTIONRESPONSE.fields_by_name['jobManagerResponseHeader'].message_type = _JOBMANAGERRESPONSEHEADER
_SETELEMENTCORRECTIONRESPONSE.fields_by_name['correction'].message_type = qm_dot_pb_dot_general__messages__pb2._MATRIX
_JOBMANAGERRESPONSEHEADER.fields_by_name['jobManagerErrorType'].enum_type = qm_dot_pb_dot_errors__pb2._JOBMANAGERERRORTYPES
_JOBMANAGERRESPONSEHEADER.fields_by_name['jobErrorDetails'].message_type = _JOBERRORDETAILS
_JOBERRORDETAILS.fields_by_name['jobOperationSpecificErrorType'].enum_type = qm_dot_pb_dot_errors__pb2._JOBOPERATIONSPECIFICERRORTYPES
_JOBERRORDETAILS.fields_by_name['configQueryErrorType'].enum_type = qm_dot_pb_dot_errors__pb2._CONFIGQUERYERRORTYPES
_JOBERRORDETAILS.oneofs_by_name['errorType'].fields.append(
  _JOBERRORDETAILS.fields_by_name['jobOperationSpecificErrorType'])
_JOBERRORDETAILS.fields_by_name['jobOperationSpecificErrorType'].containing_oneof = _JOBERRORDETAILS.oneofs_by_name['errorType']
_JOBERRORDETAILS.oneofs_by_name['errorType'].fields.append(
  _JOBERRORDETAILS.fields_by_name['configQueryErrorType'])
_JOBERRORDETAILS.fields_by_name['configQueryErrorType'].containing_oneof = _JOBERRORDETAILS.oneofs_by_name['errorType']
_JOBOPERATIONSPECIFICERROR.fields_by_name['type'].enum_type = qm_dot_pb_dot_errors__pb2._JOBOPERATIONSPECIFICERRORTYPES
DESCRIPTOR.message_types_by_name['GetElementCorrectionRequest'] = _GETELEMENTCORRECTIONREQUEST
DESCRIPTOR.message_types_by_name['GetElementCorrectionResponse'] = _GETELEMENTCORRECTIONRESPONSE
DESCRIPTOR.message_types_by_name['SetElementCorrectionRequest'] = _SETELEMENTCORRECTIONREQUEST
DESCRIPTOR.message_types_by_name['SetElementCorrectionResponse'] = _SETELEMENTCORRECTIONRESPONSE
DESCRIPTOR.message_types_by_name['JobManagerResponseHeader'] = _JOBMANAGERRESPONSEHEADER
DESCRIPTOR.message_types_by_name['JobErrorDetails'] = _JOBERRORDETAILS
DESCRIPTOR.message_types_by_name['JobOperationSpecificError'] = _JOBOPERATIONSPECIFICERROR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetElementCorrectionRequest = _reflection.GeneratedProtocolMessageType('GetElementCorrectionRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETELEMENTCORRECTIONREQUEST,
  '__module__' : 'qm.pb.job_manager_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.jobManager.GetElementCorrectionRequest)
  })
_sym_db.RegisterMessage(GetElementCorrectionRequest)

GetElementCorrectionResponse = _reflection.GeneratedProtocolMessageType('GetElementCorrectionResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETELEMENTCORRECTIONRESPONSE,
  '__module__' : 'qm.pb.job_manager_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.jobManager.GetElementCorrectionResponse)
  })
_sym_db.RegisterMessage(GetElementCorrectionResponse)

SetElementCorrectionRequest = _reflection.GeneratedProtocolMessageType('SetElementCorrectionRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETELEMENTCORRECTIONREQUEST,
  '__module__' : 'qm.pb.job_manager_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.jobManager.SetElementCorrectionRequest)
  })
_sym_db.RegisterMessage(SetElementCorrectionRequest)

SetElementCorrectionResponse = _reflection.GeneratedProtocolMessageType('SetElementCorrectionResponse', (_message.Message,), {
  'DESCRIPTOR' : _SETELEMENTCORRECTIONRESPONSE,
  '__module__' : 'qm.pb.job_manager_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.jobManager.SetElementCorrectionResponse)
  })
_sym_db.RegisterMessage(SetElementCorrectionResponse)

JobManagerResponseHeader = _reflection.GeneratedProtocolMessageType('JobManagerResponseHeader', (_message.Message,), {
  'DESCRIPTOR' : _JOBMANAGERRESPONSEHEADER,
  '__module__' : 'qm.pb.job_manager_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.jobManager.JobManagerResponseHeader)
  })
_sym_db.RegisterMessage(JobManagerResponseHeader)

JobErrorDetails = _reflection.GeneratedProtocolMessageType('JobErrorDetails', (_message.Message,), {
  'DESCRIPTOR' : _JOBERRORDETAILS,
  '__module__' : 'qm.pb.job_manager_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.jobManager.JobErrorDetails)
  })
_sym_db.RegisterMessage(JobErrorDetails)

JobOperationSpecificError = _reflection.GeneratedProtocolMessageType('JobOperationSpecificError', (_message.Message,), {
  'DESCRIPTOR' : _JOBOPERATIONSPECIFICERROR,
  '__module__' : 'qm.pb.job_manager_pb2'
  # @@protoc_insertion_point(class_scope:qm.grpc.jobManager.JobOperationSpecificError)
  })
_sym_db.RegisterMessage(JobOperationSpecificError)


DESCRIPTOR._options = None

_JOBMANAGERSERVICE = _descriptor.ServiceDescriptor(
  name='JobManagerService',
  full_name='qm.grpc.jobManager.JobManagerService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1112,
  serialized_end=1381,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetElementCorrection',
    full_name='qm.grpc.jobManager.JobManagerService.GetElementCorrection',
    index=0,
    containing_service=None,
    input_type=_GETELEMENTCORRECTIONREQUEST,
    output_type=_GETELEMENTCORRECTIONRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetElementCorrection',
    full_name='qm.grpc.jobManager.JobManagerService.SetElementCorrection',
    index=1,
    containing_service=None,
    input_type=_SETELEMENTCORRECTIONREQUEST,
    output_type=_SETELEMENTCORRECTIONRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_JOBMANAGERSERVICE)

DESCRIPTOR.services_by_name['JobManagerService'] = _JOBMANAGERSERVICE

# @@protoc_insertion_point(module_scope)



from qm.pb.inc_qua_pb2 import QuaProgram

from qm._loc import _get_loc


class StatementsCollection(object):
    def __init__(self, body):
        super(StatementsCollection, self).__init__()
        self._body = body

    def play(
        self,
        pulse,
        element,
        duration=None,
        condition=None,
        target="",
        chirp=None,
        truncate=None,
    ):
        """
        Play a pulse to a quantum element as per the OPX config
        :param pulse: A tuple (pulse, amp). pulse is string of pulse name, amp is a 4 matrix
        :param element:
        :param duration:
        :param condition:
        :param target:
        :return:
        """

        amp = None
        if type(pulse) is tuple:
            pulse, amp = pulse

        loc = _get_loc()
        statement = self._body.statements.add()
        statement.play.SetInParent()
        statement.play.loc = loc
        if isinstance(pulse, QuaProgram.RampPulse):
            statement.play.rampPulse.SetInParent()
            statement.play.rampPulse.CopyFrom(pulse)
        else:
            statement.play.namedPulse.SetInParent()
            statement.play.namedPulse.name = pulse
            statement.play.pulse.name = pulse
        statement.play.qe.name = element
        statement.play.targetInput = target
        if duration is not None:
            statement.play.duration.CopyFrom(duration)
        if condition is not None:
            statement.play.condition.CopyFrom(condition)
        if chirp is not None:
            statement.play.chirp.CopyFrom(chirp)
            statement.play.chirp.loc = loc
        if amp is not None:
            statement.play.amp.SetInParent()
            statement.play.amp.loc = loc
            statement.play.amp.v0.CopyFrom(amp[0])
            for i in range(1, 4, 1):
                if amp[i] is not None:
                    getattr(statement.play.amp, "v" + str(i)).CopyFrom(amp[i])
        if truncate is not None:
            statement.play.truncate.CopyFrom(truncate)

    def pause(self, *elements):
        """
        Pause the execution of the given quantum elements
        :param elements:
        :return:
        """
        statement = self._body.statements.add()
        statement.pause.SetInParent()
        statement.pause.loc = _get_loc()
        for element in elements:
            element_ref = statement.pause.qe.add()
            element_ref.name = element

    def update_frequency(self, element, new_frequency, units, keep_phase):
        """
        Updates the frequency of a given Quantum Element
        :param element: The quantum element to set the frequency to
        :param new_frequency: The new frequency value to set
        :return:
        """
        statement = self._body.statements.add()
        statement.updateFrequency.SetInParent()
        statement.updateFrequency.loc = _get_loc()
        statement.updateFrequency.qe.name = element
        if units == "Hz":
            statement.updateFrequency.units = QuaProgram.UpdateFrequencyStatement.Hz
        elif units == "mHz":
            statement.updateFrequency.units = QuaProgram.UpdateFrequencyStatement.mHz
        elif units == "uHz":
            statement.updateFrequency.units = QuaProgram.UpdateFrequencyStatement.uHz
        elif units == "nHz":
            statement.updateFrequency.units = QuaProgram.UpdateFrequencyStatement.nHz
        elif units == "pHz":
            statement.updateFrequency.units = QuaProgram.UpdateFrequencyStatement.pHz
        else:
            raise RuntimeError(f'unknown units "{units}"')
        statement.updateFrequency.keepPhase = keep_phase
        statement.updateFrequency.value.CopyFrom(new_frequency)

    def update_correction(self, element, c00, c01, c10, c11):
        """
        Updates the correction of a given Quantum Element
        :param element: The quantum element to set the correction to
        :param c00:
        :param c01:
        :param c10:
        :param c11:
        :return:
        """
        statement = self._body.statements.add()
        statement.updateCorrection.SetInParent()
        statement.updateCorrection.qe.name = element
        statement.updateCorrection.correction.SetInParent()
        statement.updateCorrection.correction.c0.CopyFrom(c00)
        statement.updateCorrection.correction.c1.CopyFrom(c01)
        statement.updateCorrection.correction.c2.CopyFrom(c10)
        statement.updateCorrection.correction.c3.CopyFrom(c11)

    def align(self, *elements):
        """
        Align the given quantum elements
        :param elements:
        :return:
        """
        statement = self._body.statements.add()
        statement.align.SetInParent()
        statement.align.loc = _get_loc()
        for element in elements:
            element_ref = statement.align.qe.add()
            element_ref.name = element

    def reset_phase(self, element):
        """
        TODO: document
        :param element:
        :return:
        """
        statement = self._body.statements.add()
        statement.resetPhase.SetInParent()
        statement.resetPhase.qe.SetInParent()
        statement.resetPhase.qe.name = element

    def wait(self, duration, *elements):
        """
        Waits for the given duration on all provided quantum elements
        :param duration:
        :param elements:
        :return:
        """
        statement = self._body.statements.add()
        statement.wait.SetInParent()
        statement.wait.loc = _get_loc()
        statement.wait.time.CopyFrom(duration)
        for element in elements:
            element_ref = statement.wait.qe.add()
            element_ref.name = element

    def wait_for_trigger(self, pulse_to_play, *elements):
        statement = self._body.statements.add()
        statement.waitForTrigger.SetInParent()
        statement.waitForTrigger.loc = _get_loc()
        if pulse_to_play is not None:
            statement.waitForTrigger.pulseToPlay.name = pulse_to_play
        for element in elements:
            element_ref = statement.waitForTrigger.qe.add()
            element_ref.name = element

    def save(self, source, result):
        statement = self._body.statements.add()
        statement.save.SetInParent()
        statement.save.loc = _get_loc()
        statement.save.source.CopyFrom(source)
        statement.save.tag = result._get_var_name()

    def z_rotation(self, angle, *elements):
        for element in elements:
            statement = self._body.statements.add()
            statement.zRotation.SetInParent()
            statement.zRotation.loc = _get_loc()
            statement.zRotation.value.CopyFrom(angle)
            statement.zRotation.qe.name = element

    def reset_frame(self, *elements):
        for element in elements:
            statement = self._body.statements.add()
            statement.resetFrame.SetInParent()
            statement.resetFrame.loc = _get_loc()
            statement.resetFrame.qe.name = element

    def ramp_to_zero(self, element, duration):
        statement = self._body.statements.add()
        statement.rampToZero.SetInParent()
        statement.rampToZero.qe.SetInParent()
        statement.rampToZero.qe.name = element
        if duration is not None:
            statement.rampToZero.duration.SetInParent()
            statement.rampToZero.duration.value = duration

    def measure(self, pulse, element, stream=None, *processes):
        """
        Measure a quantum element using the given pulse, process the result with the integration weights and
        store the results to the provided variables
        :param pulse:
        :param element:
        :param stream:
        :type stream: _ResultSource
        :param processes: an iterable of analog processes
        :return:
        """
        amp = None
        if type(pulse) is tuple:
            pulse, amp = pulse

        loc = _get_loc()
        statement = self._body.statements.add()
        statement.measure.SetInParent()
        statement.measure.loc = loc
        statement.measure.pulse.name = pulse
        statement.measure.qe.name = element
        if stream is not None:
            statement.measure.streamAs = stream._get_var_name()

        for analog_process in processes:
            added_process = statement.measure.measureProcesses.add()
            added_process.CopyFrom(analog_process)

        if amp is not None:
            statement.measure.amp.SetInParent()
            statement.measure.amp.loc = loc
            statement.measure.amp.v0.CopyFrom(amp[0])
            for i in range(1, 4, 1):
                if amp[i] is not None:
                    getattr(statement.measure.amp, "v" + str(i)).CopyFrom(amp[i])

    def if_block(self, condition, unsafe=False):
        statement = self._body.statements.add()
        ifstatement = getattr(statement, "if")
        ifstatement.SetInParent()
        ifstatement.loc = _get_loc()
        ifstatement.condition.CopyFrom(condition)
        ifstatement.unsafe = unsafe
        ifstatement.body.SetInParent()
        return StatementsCollection(ifstatement.body)

    def for_each(self, iterators):
        statement = self._body.statements.add()
        forEach = statement.forEach
        forEach.SetInParent()
        forEach.loc = _get_loc()
        for it in iterators:
            pb_it = forEach.iterator.add()
            pb_it.variable.CopyFrom(it[0])
            pb_it.array.CopyFrom(it[1])
        forEach.body.SetInParent()
        return StatementsCollection(forEach.body)

    def get_last_statement(self):
        statements = self._body.statements
        l = len(statements)
        if l == 0:
            return None
        return statements[l - 1]

    def for_block(self):
        statement = self._body.statements.add()
        forstatement = getattr(statement, "for")
        forstatement.SetInParent()
        forstatement.loc = _get_loc()
        return forstatement

    def qrun_block(self):
        statement = self._body.statements.add()
        qrun_statement = statement.qrun
        qrun_statement.SetInParent()
        qrun_statement.loc = _get_loc()
        return qrun_statement

    def assign(self, target, expression):
        """
        Assign a value calculated by :expression into :target
        :param target: The name of the variable to assign to
        :param expression: The expression to calculate
        :return:
        """
        statement = self._body.statements.add()
        statement.assign.SetInParent()
        statement.assign.loc = _get_loc()
        statement.assign.target.CopyFrom(target)
        statement.assign.expression.CopyFrom(expression)



from qm.pb.frontend_pb2 import SimulationRequest
from qm.simulate.interface import SimulatorInterface


class RawInterface(SimulatorInterface):
    """
    Creates a raw interface for simulation

    :param list connections:

        List of tuples with loopback connections. Each tuple can be:

            1. Physical connection between port and samples:

                ``(fromController: str, fromPort: int, toSamples: List[float])``
    """

    def __init__(self, connections, noisePower=0.0):
        if (
            type(noisePower) is not float and type(noisePower) is not int
        ) or noisePower < 0:
            raise Exception("noisePower must be a positive number")

        self.noisePower = noisePower

        if type(connections) is not list:
            raise Exception("connections argument must be of type list")

        self.connections = list()
        for connection in connections:
            if type(connection) is not tuple:
                raise Exception("each connection must be of type tuple")
            if len(connection) == 3:
                if (
                    type(connection[0]) is not str
                    or type(connection[1]) is not int
                    or type(connection[2]) is not list
                ):
                    raise Exception(
                        "connection should be (fromController, fromPort, toSamples)"
                    )
                self.connections.append(connection)
            else:
                raise Exception("connection should be tuple of length 3 or 4")

    def update_simulate_request(self, request: SimulationRequest):
        request.simulate.simulationInterface.raw.SetInParent()
        request.simulate.simulationInterface.raw.noisePower = self.noisePower
        for connection in self.connections:
            con = request.simulate.simulationInterface.raw.connections.add()
            con.fromController = connection[0]
            con.fromPort = connection[1]
            con.toSamples.extend(connection[2])
